﻿using System.Diagnostics.CodeAnalysis;
using System.Reflection;

namespace NQueries.Utils
{
    /// <summary>
    /// Defines extension methods for <see cref="MemberInfo"/>.
    /// </summary>
    internal static class MemberInfoExtensions
    {
        /// <summary>
        /// Gets the attribute from the <see cref="MemberInfo"/>.
        /// </summary>
        /// <typeparam name="TAttribute"></typeparam>
        /// <param name="memberInfo"></param>
        /// <returns></returns>
        [SuppressMessage("Microsoft.Design", "CA1004:GenericMethodsShouldProvideTypeParameter")]
        public static TAttribute GetAttributeOf<TAttribute>(this MemberInfo memberInfo)
        {
            Check.IsNotNull(memberInfo, "memberInfo");
            object[] attributes = memberInfo.GetCustomAttributes(typeof(TAttribute), true);
            if (attributes.Length == 0)
                return default(TAttribute);
            return (TAttribute)attributes[0];
        }
    }
}